@@ -227,40 +227,46 @@ evaluated as a PROGN."
               (examine var keyword))))
         (nreverse caller-dxable)))))
 
-(sb-xc:defmacro defun (&environment env name lambda-list &body body)
-  "Define a function at top level."
-  (check-designator name defun)
-  #+sb-xc-host
-  (unless (cl:symbol-package (fun-name-block-name name))
-    (warn "DEFUN of uninterned function name ~S (tricky for GENESIS)" name))
+(flet ((defun-expander (env name lambda-list body snippet)
   (multiple-value-bind (forms decls doc) (parse-body body t)
     ;; Maybe kill docstring, but only under the cross-compiler.
-    #!+(and (not sb-doc) (host-feature sb-xc-host)) (setq doc nil)
+    #+(and (not sb-doc) sb-xc-host) (setq doc nil)
     (let* (;; stuff shared between LAMBDA and INLINE-LAMBDA and NAMED-LAMBDA
            (lambda-guts `(,@decls (block ,(fun-name-block-name name) ,@forms)))
            (lambda `(lambda ,lambda-list ,@lambda-guts))
            (named-lambda `(named-lambda ,name ,lambda-list
                            ,@(when *top-level-form-p* '((declare (sb-c::top-level-form))))
                            ,@(when doc (list doc)) ,@lambda-guts))
-           (dxable-args (extract-dx-args lambda-list decls))
+           ;; DXABLE-ARGS and SNIPPET are mutually exclusive, so we can sleazily pass
+           ;; whichever exists (if either does) as one parameter to %DEFUN.
+           (extra-info (or snippet (extract-dx-args lambda-list decls)))
            (inline-thing
-            (or (sb-kernel::defstruct-generated-defn-p name lambda-list body)
-                (when (save-inline-expansion-p name)
+            (cond ((member snippet '(:predicate :copier :accessor)) nil)
+                  ;; If the defstruct snippet is :CONSTRUCTOR, we might have to store
+                  ;; a full inline expansion depending on the lexical environment.
+                  ((save-inline-expansion-p name)
                   ;; we want to attempt to inline, so complain if we can't
-                  (acond ((sb-c:maybe-inline-syntactic-closure lambda env)
-                          (list 'quote it))
+                   (cond ((sb-c:maybe-inline-syntactic-closure lambda env))
                          (t
                           (#+sb-xc-host warn
                            #-sb-xc-host sb-c:maybe-compiler-notify
                            "lexical environment too hairy, can't inline DEFUN ~S"
                            name)
                           nil))))))
+      (when (and (eq snippet :constructor)
+                 (not (typep inline-thing '(cons (eql sb-c:lambda-with-lexenv)))))
+        ;; constructor in null lexenv need not save the expansion
+        (setq inline-thing nil))
+      (when inline-thing
+        (setq inline-thing (list 'quote inline-thing)))
+      (when (and extra-info (not (keywordp extra-info)))
+        (setq extra-info (list 'quote extra-info)))
       `(progn
          (eval-when (:compile-toplevel)
-           (sb-c:%compiler-defun ',name t ,inline-thing ',dxable-args))
+           (sb-c:%compiler-defun ',name t ,inline-thing ,extra-info))
          (%defun ',name ,named-lambda
-                 ,@(when (or inline-thing dxable-args) (list inline-thing))
-                 ,@(when dxable-args `(',dxable-args)))
+                 ,@(when (or inline-thing extra-info) `(,inline-thing))
+                 ,@(when extra-info `(,extra-info)))
          ;; This warning, if produced, comes after the DEFUN happens.
          ;; When compiling, there's no real difference, but when interpreting,
          ;; if there is a handler for style-warning that nonlocally exits,
@@ -269,8 +275,19 @@ evaluated as a PROGN."
          ,@(when (typep name '(cons (eql setf)))
              `((eval-when (:compile-toplevel :execute)
                  (sb-c::warn-if-setf-macro ',name))
-               ',name))))))
+               ',name)))))))
 
+  (sb-xc:defmacro defun (&environment env name lambda-list &body body)
+    "Define a function at top level."
+    (check-designator name defun)
+    #+sb-xc-host
+    (unless (cl:symbol-package (fun-name-block-name name))
+      (warn "DEFUN of uninterned function name ~S (tricky for GENESIS)" name))
+    (defun-expander env name lambda-list body nil))
+
+  ;; extended defun as used by defstruct
+  (sb-xc:defmacro sb-c:xdefun (&environment env name snippet lambda-list &body body)
+    (defun-expander env name lambda-list body snippet)))
 
 ;;;; DEFVAR and DEFPARAMETER
 
@@ -281,14 +298,19 @@ evaluated as a PROGN."
   documentation string for the variable."
   (check-designator var defvar)
   ;; Maybe kill docstring, but only under the cross-compiler.
-  #!+(and (not sb-doc) (host-feature sb-xc-host)) (setq doc nil)
+  #+(and (not sb-doc) sb-xc-host) (setq doc nil)
   `(progn
      (eval-when (:compile-toplevel)
        (%compiler-defvar ',var))
      (%defvar ',var
               (sb-c:source-location)
-              ,@(and valp
-                     `((unless (%boundp ',var) ,val)))
+              ,@(cond ((not valp)
+                       nil)
+                      ((constantp val)
+                       ;; No need to avoid evaluation if it's a constant.
+                       `(',(constant-form-value val)))
+                      (val
+                       `((unless (%boundp ',var) ,val))))
               ,@(and docp
                      `(',doc)))))
 
@@ -300,7 +322,7 @@ evaluated as a PROGN."
   string for the parameter."
   (check-designator var defparameter)
   ;; Maybe kill docstring, but only under the cross-compiler.
-  #!+(and (not sb-doc) (host-feature sb-xc-host)) (setq doc nil)
+  #+(and (not sb-doc) sb-xc-host) (setq doc nil)
   `(progn
      (eval-when (:compile-toplevel)
        (%compiler-defvar ',var))
@@ -805,9 +827,9 @@ condition."
   ;; or a symbol naming a condition class at compile time,
   ;; and HANDLER must be a global function specified as either 'F or #'F.
   `(%handler-bind ,bindings
-                  #!-x86 (progn ,@forms)
+                  #-x86 (progn ,@forms)
                   ;; Need to catch FP errors here!
-                  #!+x86 (multiple-value-prog1 (progn ,@forms) (float-wait))))
+                  #+x86 (multiple-value-prog1 (progn ,@forms) (float-wait))))
 
 (sb-xc:defmacro handler-case (form &rest cases)
   "(HANDLER-CASE form { (type ([var]) body) }* )
@@ -845,9 +867,9 @@ specification."
                          cases)))
           (with-unique-names (block cell form-fun)
             `(dx-flet ((,form-fun ()
-                         #!-x86 (progn ,form) ;; no declarations are accepted
+                         #-x86 (progn ,form) ;; no declarations are accepted
                          ;; Need to catch FP errors here!
-                         #!+x86 (multiple-value-prog1 ,form (float-wait)))
+                         #+x86 (multiple-value-prog1 ,form (float-wait)))
                        ,@(reverse local-funs))
                (declare (optimize (sb-c::check-tag-existence 0)))
                (block ,block
