@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 (deftype byte-buffer () '(vector (unsigned-byte 8)))
 (defvar *byte-buffer*)
@@ -23,8 +23,7 @@
 
 (deftype location-kind ()
   '(member :unknown-return :known-return :internal-error :non-local-exit
-           :block-start :call-site :single-value-return :non-local-entry
-           :step-before-vop))
+           :block-start :call-site :single-value-return :non-local-entry))
 
 ;;; The LOCATION-INFO structure holds the information what we need
 ;;; about locations which code generation decided were "interesting".
@@ -44,7 +43,6 @@
             (:copier nil))
   (label nil :type (or null label))
   (tn nil :type (or null tn) :read-only t))
-(!set-load-form-method restart-location (:xc :target) :ignore-it)
 
 ;;; This is called during code generation in places where there is an
 ;;; "interesting" location: someplace where we are likely to end up
@@ -79,7 +77,7 @@
                          (- posn (segment-header-skew segment))))))
   (values))
 
-#!-sb-fluid (declaim (inline ir2-block-physenv))
+#-sb-fluid (declaim (inline ir2-block-physenv))
 (defun ir2-block-physenv (2block)
   (declare (type ir2-block 2block))
   (block-physenv (ir2-block-block 2block)))
@@ -137,6 +135,8 @@
 ;;; The PC for the location most recently dumped.
 (defvar *previous-location*)
 (declaim (type index *previous-location*))
+(defvar *previous-live*)
+(defvar *previous-form-number*)
 
 (defun encode-restart-location (location x)
   (typecase x
@@ -144,7 +144,7 @@
      (let ((offset (- (label-position (restart-location-label x))
                       location))
            (tn (restart-location-tn x))
-           (registers-size #.(integer-length (sb-size (sb-or-lose 'sb!vm::registers)))))
+           (registers-size #.(integer-length (sb-size (sb-or-lose 'sb-vm::registers)))))
        (if tn
            (the fixnum (logior (ash offset registers-size)
                                (tn-offset tn)))
@@ -154,8 +154,8 @@
 
 (defun decode-restart-location (x)
   (declare (fixnum x))
-  (let ((registers-size #.(integer-length (sb-size (sb-or-lose 'sb!vm::registers)))))
-    (values (make-sc+offset sb!vm:descriptor-reg-sc-number
+  (let ((registers-size #.(integer-length (sb-size (sb-or-lose 'sb-vm::registers)))))
+    (values (make-sc+offset sb-vm:descriptor-reg-sc-number
                             (ldb (byte registers-size 0) x))
             (ash x (- registers-size)))))
 
@@ -170,7 +170,6 @@
            (type (or label index) label)
            (type location-kind kind)
            (type hash-table var-locs) (type (or vop null) vop))
-
   (let* ((byte-buffer *byte-buffer*)
          (stepping (and (combination-p node)
                         (combination-step-info node)))
@@ -178,8 +177,12 @@
                     (compute-live-vars live node block var-locs vop)))
          (anything-alive (and live
                               (find 1 live)))
+         (equal-live (and anything-alive
+                          (equal live *previous-live*)))
+
          (path (node-source-path node))
-         (loc (if (fixnump label) label (label-position label))))
+         (loc (if (fixnump label) label (label-position label)))
+         (form-number (source-path-form-number path)))
     (vector-push-extend
      (logior
       (if context
@@ -188,22 +191,37 @@
       (if stepping
           compiled-code-location-stepping
           0)
-      (if anything-alive
-          compiled-code-location-live
-          0)
-      (if (zerop (source-path-form-number path))
+      (if (zerop form-number)
           compiled-code-location-zero-form-number
           0)
+      (cond (equal-live
+             (cond ((eql form-number *previous-form-number*)
+                    ;; Repurpose this bit for equal-form-number since
+                    ;; -equal-live and -live don't need to appear at the
+                    ;; same time.
+                    (setf form-number 0)
+                    compiled-code-location-live)
+                   (t
+                    0)))
+            (anything-alive
+             compiled-code-location-live)
+            (t
+             0))
+      (if equal-live
+          compiled-code-location-equal-live
+          0)
       (position-or-lose kind +compiled-code-location-kinds+))
      byte-buffer)
-
     (write-var-integer (- loc *previous-location*) byte-buffer)
     (setq *previous-location* loc)
 
-    (unless (zerop (source-path-form-number path))
-      (write-var-integer (source-path-form-number path) byte-buffer))
+    (unless (zerop form-number)
+      (setf *previous-form-number* form-number)
+      (write-var-integer form-number byte-buffer))
 
-    (when anything-alive
+    (when (and anything-alive
+               (not equal-live))
+      (setf *previous-live* live)
       (write-packed-bit-vector live byte-buffer))
     (when stepping
       (write-var-string stepping byte-buffer))
@@ -252,6 +270,8 @@
 (defun compute-debug-blocks (fun var-locs)
   (declare (type clambda fun) (type hash-table var-locs))
   (let ((*previous-location* 0)
+        *previous-live*
+        *previous-form-number*
         (physenv (lambda-physenv fun))
         (byte-buffer *byte-buffer*)
         prev-block
@@ -276,12 +296,8 @@
     (when elsewhere-locations
       (dolist (loc (nreverse elsewhere-locations))
         (dump-location-from-info loc var-locs)))
-    (let ((compressed
-           (lz-compress (coerce byte-buffer
-                                '(simple-array (unsigned-byte 8) (*))))))
-      (logically-readonlyize
-       (!make-specialized-array (length compressed) '(unsigned-byte 8)
-                                compressed)))))
+    ;; lz-compress accept any array of octets and returns a simple-array
+    (logically-readonlyize (lz-compress byte-buffer))))
 
 ;;; Return DEBUG-SOURCE structure containing information derived from
 ;;; INFO.
@@ -289,7 +305,7 @@
   (declare (type source-info info))
   (let ((file-info (get-toplevelish-file-info info)))
     (multiple-value-call
-        (if function #'sb!c::make-core-debug-source #'make-debug-source)
+        (if function #'sb-c::make-core-debug-source #'make-debug-source)
      :compiled (source-info-start-time info)
      :namestring (or *source-namestring*
                      (make-file-info-namestring
@@ -350,16 +366,16 @@
       (if (zerop length)
           #()
           (logically-readonlyize
-           (!make-specialized-array length
-                                    (smallest-element-type (max max-positive
-                                                                (1- max-negative))
-                                                           (plusp max-negative))
-                                    seq))))))
+           (sb-xc:coerce seq
+                         `(simple-array
+                            ,(smallest-element-type (max max-positive
+                                                         (1- max-negative))
+                                                    (plusp max-negative))
+                            1)))))))
 
 (defun compact-vector (sequence)
   (cond ((and (= (length sequence) 1)
-              (not (typep (elt sequence 0) '(and vector
-                                             (not string)))))
+              (not (typep (elt sequence 0) '(and vector (not string)))))
          (elt sequence 0))
         (t
          (coerce-to-smallest-eltype sequence))))
@@ -391,11 +407,10 @@
 ;;; environment live and is an argument. If a :DEBUG-ENVIRONMENT TN,
 ;;; then we also exclude set variables, since the variable is not
 ;;; guaranteed to be live everywhere in that case.
-(defun dump-1-var (fun var tn minimal buffer &optional same-name-p)
+(defun dump-1-var (fun var tn minimal buffer &optional name same-name-p)
   (declare (type lambda-var var) (type (or tn null) tn)
            (type clambda fun))
-  (let* ((name (leaf-debug-name var))
-         (save-tn (and tn (tn-save-tn tn)))
+  (let* ((save-tn (and tn (tn-save-tn tn)))
          (kind (and tn (tn-kind tn)))
          (flags 0)
          (info (lambda-var-arg-info var))
@@ -431,7 +446,7 @@
     (when (and same-name-p
                (not (or more minimal)))
       (setf flags (logior flags compiled-debug-var-same-name-p)))
-    #!+64-bit ; FIXME: fails if SB-VM:N-FIXNUM-TAG-BITS is 3
+    #+64-bit ; FIXME: fails if SB-VM:N-FIXNUM-TAG-BITS is 3
               ; which early-vm.lisp claims to work
     (cond (indirect
            (setf (ldb (byte 27 8) flags) (tn-sc+offset tn))
@@ -453,20 +468,20 @@
       ;; and PARSE-COMPILED-DEBUG-VARS can create the interned symbol.
       ;; This reduces core size by omitting zillions of symbols whose names
       ;; are spelled the same.
-      (vector-push-extend (if (symbol-package name) name (string name)) buffer))
+      (vector-push-extend (if (cl:symbol-package name) name (string name)) buffer))
 
     (cond (indirect
            ;; Indirect variables live in the parent frame, and are
            ;; accessed through a saved frame pointer.
            ;; The first one/two sc-offsets are for the frame pointer,
            ;; the third is for the stack offset.
-           #!-64-bit
+           #-64-bit
            (vector-push-extend (tn-sc+offset tn) buffer)
-           #!-64-bit
+           #-64-bit
            (when save-tn
              (vector-push-extend (tn-sc+offset save-tn) buffer))
            (vector-push-extend (tn-sc+offset (leaf-info var)) buffer))
-          #!-64-bit
+          #-64-bit
           (t
            (if (and tn (tn-offset tn))
                (vector-push-extend (tn-sc+offset tn) buffer)
@@ -475,6 +490,18 @@
              (vector-push-extend (tn-sc+offset save-tn) buffer)))))
   (values))
 
+(defun leaf-principal-name (leaf)
+  ;; If all the references are from the same substituted variable
+  ;; use its name.
+  ;; Helps with &key processing variables.
+  (let ((refs (leaf-refs leaf)))
+    (loop for ref in refs
+          for name = (ref-%source-name ref)
+          for first-name = name then first-name
+          unless (eq name first-name)
+          return (leaf-debug-name leaf)
+          finally (return name))))
+
 ;;; Return a vector suitable for use as the DEBUG-FUN-VARS
 ;;; of FUN. LEVEL is the current DEBUG-INFO quality. VAR-LOCS is a
 ;;; hash table in which we enter the translation from LAMBDA-VARS to
@@ -484,10 +511,10 @@
   (declare (type clambda fun) (type hash-table var-locs))
   (collect ((vars))
     (labels ((frob-leaf (leaf tn gensym-p)
-               (let ((name (leaf-debug-name leaf)))
+               (let ((name (leaf-principal-name leaf)))
                  (when (and name (leaf-refs leaf) (tn-offset tn)
-                            (or gensym-p (symbol-package name)))
-                   (vars (cons leaf tn)))))
+                            (or gensym-p (cl:symbol-package name)))
+                   (vars (list* name leaf tn)))))
              (frob-lambda (x gensym-p)
                (dolist (leaf (lambda-vars x))
                  (frob-leaf leaf (leaf-info leaf) gensym-p))))
@@ -503,21 +530,21 @@
 
     (let ((sorted (sort (vars) #'string<
                         :key (lambda (x)
-                               (symbol-name (leaf-debug-name (car x))))))
+                               (symbol-name (car x)))))
           (prev-name nil)
           (i 0)
           (buffer (make-array 0 :fill-pointer 0 :adjustable t))
           ;; XEPs don't have any useful variables
           (minimal (eq (functional-kind fun) :external)))
       (declare (type index i))
-      (dolist (x sorted)
-        (let* ((var (car x))
-               (name (leaf-debug-name var)))
-          (dump-1-var fun var (cdr x) minimal buffer
-                      (and prev-name (eq prev-name name)))
-          (setf prev-name name)
-          (setf (gethash var var-locs) i)
-          (incf i)))
+      (loop for (name var . tn) in sorted
+            do
+            (dump-1-var fun var tn minimal buffer
+                        name
+                        (and prev-name (eq prev-name name)))
+            (setf prev-name name)
+            (setf (gethash var var-locs) i)
+            (incf i))
       (compact-vector buffer))))
 
 ;;; Return a vector suitable for use as the DEBUG-FUN-VARS of
@@ -625,24 +652,26 @@
                    name)))
     (funcall (compiled-debug-fun-ctor kind)
              :name name
-             #!-fp-and-pc-standard-save :return-pc
-             #!-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc 2env))
-             #!-fp-and-pc-standard-save :return-pc-pass
-             #!-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc-pass 2env))
-             #!-fp-and-pc-standard-save :old-fp
-             #!-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-old-fp 2env))
+             #-fp-and-pc-standard-save :return-pc
+             #-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc 2env))
+             #-fp-and-pc-standard-save :return-pc-pass
+             #-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc-pass 2env))
+             #-fp-and-pc-standard-save :old-fp
+             #-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-old-fp 2env))
              :encoded-locs
              (cdf-encode-locs
               (label-position (ir2-physenv-environment-start 2env))
               (label-position (ir2-physenv-elsewhere-start 2env))
+              (source-path-form-number (node-source-path (lambda-bind fun)))
+              (label-position (block-label (lambda-block fun)))
               (when (ir2-physenv-closure-save-tn 2env)
                 (tn-sc+offset (ir2-physenv-closure-save-tn 2env)))
-              #!+unwind-to-frame-and-call-vop
+              #+unwind-to-frame-and-call-vop
               (when (ir2-physenv-bsp-save-tn 2env)
                 (tn-sc+offset (ir2-physenv-bsp-save-tn 2env)))
-              #!-fp-and-pc-standard-save
+              #-fp-and-pc-standard-save
               (label-position (ir2-physenv-lra-saved-pc 2env))
-              #!-fp-and-pc-standard-save
+              #-fp-and-pc-standard-save
               (label-position (ir2-physenv-cfp-saved-pc 2env))))))
 
 ;;; Return a complete C-D-F structure for FUN. This involves
@@ -652,7 +681,7 @@
   (declare (type clambda fun) (type hash-table var-locs))
   (let* ((dfun (dfun-from-fun fun))
          (actual-level (policy (lambda-bind fun) compute-debug-fun))
-         (level (cond #!+sb-dyncount
+         (level (cond #+sb-dyncount
                       (*collect-dynamic-statistics*
                        (max actual-level 2))
                       (actual-level))))
@@ -669,8 +698,6 @@
                  (compute-vars fun level var-locs))
            (setf (compiled-debug-fun-arguments dfun)
                  (compute-args fun var-locs))))
-    (setf (compiled-debug-fun-form-number dfun)
-          (source-path-form-number (node-source-path (lambda-bind fun))))
     (when (>= level 1)
       (setf (compiled-debug-fun-blocks dfun)
             (compute-debug-blocks fun var-locs)))
@@ -688,20 +715,12 @@
 
 ;;;; full component dumping
 
-;;; Compute the full form (simple-vector) function map.
+;;; Compute the full form function map.
 (defun compute-debug-fun-map (sorted)
   (declare (list sorted))
-  (let* ((len (1- (* (length sorted) 2)))
-         (funs-vec (make-array len)))
-    (do ((i -1 (+ i 2))
-         (sorted sorted (cdr sorted)))
-        ((= i len))
-      (declare (fixnum i))
-      (let ((dfun (car sorted)))
-        (unless (minusp i)
-          (setf (svref funs-vec i) (car dfun)))
-        (setf (svref funs-vec (1+ i)) (cdr dfun))))
-    funs-vec))
+  (loop for (fun next) on sorted
+        do (setf (compiled-debug-fun-next fun) next))
+  (car sorted))
 
 ;;; Return a DEBUG-INFO structure describing COMPONENT. This has to be
 ;;; called after assembly so that source map information is available.
@@ -724,10 +743,8 @@
         (if component-tlf-num
             (aver (= component-tlf-num tlf-num))
             (setf component-tlf-num tlf-num))
-        (push (cons (label-position (block-label (lambda-block lambda)))
-                    (compute-1-debug-fun lambda var-locs))
-              dfuns)))
-    (let* ((sorted (sort dfuns #'< :key #'car))
+        (push (compute-1-debug-fun lambda var-locs) dfuns)))
+    (let* ((sorted (sort dfuns #'< :key #'compiled-debug-fun-offset))
            (fun-map (compute-debug-fun-map sorted)))
       (make-compiled-debug-info
        ;; COMPONENT-NAME is often not useful, and sometimes completely fubar.
@@ -735,9 +752,9 @@
        ;; so if there's only one function, pick that as the component name.
        ;; Otherwise preserve whatever crummy name was already assigned.
        :name (let* ((2comp (component-info component))
-                    (entries (sb!c::ir2-component-entries 2comp)))
+                    (entries (sb-c::ir2-component-entries 2comp)))
                (or (and (not (cdr entries))
-                        (sb!c::entry-info-name (car entries)))
+                        (sb-c::entry-info-name (car entries)))
                    (component-name component)))
        :fun-map fun-map
        :tlf-num+offset (pack-tlf-num+offset
