@@ -13,27 +13,37 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
+
+;;; An OPAQUE-BOX instance is used to pass data from IR1 to IR2 as
+;;; a quoted object in a "source form" (not user-written) such that the
+;;; contained object is in a for-evaluation position but ignored by
+;;; the compiler's constant-dumping logic. In addition to this structure
+;;; type, a few other IR1 object types are implicitly opaque.
+(defstruct (opaque-box (:constructor opaquely-quote (value))
+                       (:copier nil)
+                       (:predicate opaque-box-p))
+  value)
 
 ;;; ANSI limits on compilation
-(defconstant sb!xc:call-arguments-limit sb!xc:most-positive-fixnum
+(defconstant sb-xc:call-arguments-limit sb-xc:most-positive-fixnum
   "The exclusive upper bound on the number of arguments which may be passed
   to a function, including &REST args.")
-(defconstant sb!xc:lambda-parameters-limit sb!xc:most-positive-fixnum
+(defconstant sb-xc:lambda-parameters-limit sb-xc:most-positive-fixnum
   "The exclusive upper bound on the number of parameters which may be specified
   in a given lambda list. This is actually the limit on required and &OPTIONAL
   parameters. With &KEY and &AUX you can get more.")
-(defconstant sb!xc:multiple-values-limit sb!xc:most-positive-fixnum
+(defconstant sb-xc:multiple-values-limit sb-xc:most-positive-fixnum
   "The exclusive upper bound on the number of multiple VALUES that you can
   return.")
 
 ;;;; cross-compiler-only versions of CL special variables, so that we
 ;;;; don't have weird interactions with the host compiler
 
-(defvar sb!xc:*compile-file-pathname*)
-(defvar sb!xc:*compile-file-truename*)
-(defvar sb!xc:*compile-print*)
-(defvar sb!xc:*compile-verbose*)
+(defvar sb-xc:*compile-file-pathname*)
+(defvar sb-xc:*compile-file-truename*)
+(defvar sb-xc:*compile-print*)
+(defvar sb-xc:*compile-verbose*)
 
 ;;;; miscellaneous types used both in the cross-compiler and on the target
 
@@ -41,12 +51,15 @@
 ;;;; somewhere else, not "early-c", since they're after all not part
 ;;;; of the compiler.
 
-;;; the type of LAYOUT-DEPTHOID slot values
-(def!type layout-depthoid () '(or index (integer -1 -1)))
+;;; the type of LAYOUT-DEPTHOID and LAYOUT-LENGTH values.
+;;; Each occupies two bytes of the %BITS slot when possible,
+;;; otherwise a slot unto itself.
+(def!type layout-depthoid () '(integer -1 #x7FFF))
+(def!type layout-length () '(integer 0 #xFFFF))
 (def!type layout-bitmap ()
   ;; FIXME: Probably should exclude negative bignum
-  #!+compact-instance-header 'integer
-  #!-compact-instance-header '(and integer (not (eql 0))))
+  #+compact-instance-header 'integer
+  #-compact-instance-header '(and integer (not (eql 0))))
 
 ;;; An INLINEP value describes how a function is called. The values
 ;;; have these meanings:
@@ -99,11 +112,7 @@
 (defvar *allow-instrumenting*)
 
 ;;; miscellaneous forward declarations
-;; FIXME: this is a kludge due to the absence of a 'vop' argument
-;; to ALLOCATION-TRAMP in the x86-64 backend.
-#!+immobile-code
-(defvar *code-is-immobile*)
-#!+sb-dyncount (defvar *collect-dynamic-statistics*)
+#+sb-dyncount (defvar *collect-dynamic-statistics*)
 (defvar *component-being-compiled*)
 (defvar *compiler-error-context*)
 (defvar *compiler-error-count*)
@@ -115,10 +124,9 @@
 (defvar *current-path*)
 (defvar *current-component*)
 (defvar *delayed-ir1-transforms*)
-#!+sb-dyncount
+#+sb-dyncount
 (defvar *dynamic-counts-tn*)
 (defvar *elsewhere-label*)
-(defvar *event-info*)
 (defvar *event-note-threshold*)
 (defvar *failure-p*)
 (defvar *source-info*)
@@ -129,7 +137,6 @@
 (defvar *lambda-conversions*)
 (defvar *compile-object* nil)
 (defvar *location-context* nil)
-(defvar *msan-unpoison* nil)
 
 (defvar *stack-allocate-dynamic-extent* t
   "If true (the default), the compiler respects DYNAMIC-EXTENT declarations
@@ -138,24 +145,26 @@ possible. Potentially long (over one page in size) vectors are, however, not
 stack allocated except in zero SAFETY code, as such a vector could overflow
 the stack without triggering overflow protection.")
 
-(!begin-collecting-cold-init-forms)
 ;;; This lock is seized in the compiler, and related areas -- like the
 ;;; classoid/layout/class system.
-(defglobal **world-lock** nil)
+;;; Assigning a literal object enables genesis to dump and load it
+;;; without need of a cold-init function.
 #-sb-xc-host
-(!cold-init-forms
- (setf **world-lock** (sb!thread:make-mutex :name "World Lock")))
-(!defun-from-collected-cold-init-forms !world-lock-cold-init)
+(!define-load-time-global **world-lock** #.(sb-thread:make-mutex :name "World Lock"))
+
+#-sb-xc-host
+(define-load-time-global *static-linker-lock*
+    (sb-thread:make-mutex :name "static linker"))
 
 (defmacro with-world-lock (() &body body)
-  `(sb!thread:with-recursive-lock (**world-lock**)
-     ,@body))
+  #+sb-xc-host `(progn ,@body)
+  #-sb-xc-host `(sb-thread:with-recursive-lock (**world-lock**) ,@body))
 
 ;;; unique ID for the next object created (to let us track object
 ;;; identity even across GC, useful for understanding weird compiler
 ;;; bugs where something is supposed to be unique but is instead
 ;;; exists as duplicate objects)
-#!+sb-show
+#+sb-show
 (progn
   (defvar *object-id-counter* 0)
   (defun new-object-id ()
@@ -231,28 +240,16 @@ the stack without triggering overflow protection.")
                 :format-arguments (list symbol)))
   (values))
 
-(def!struct (debug-name-marker (:print-function print-debug-name-marker)
-                               (:copier nil)))
+(defstruct (debug-name-marker (:print-function print-debug-name-marker)
+                              ;; make these satisfy SB-XC:INSTANCEP
+                              #+sb-xc-host (:include structure!object)
+                              (:copier nil)))
 
 (defvar *debug-name-level* 4)
 (defvar *debug-name-length* 12)
 (defvar *debug-name-punt*)
-(defvar *debug-name-sharp*)
-(defvar *debug-name-ellipsis*)
-
-(defmethod make-load-form ((marker debug-name-marker) &optional env)
-  (declare (ignore env))
-  (cond ((eq marker *debug-name-sharp*)
-         `(if (boundp '*debug-name-sharp*)
-              *debug-name-sharp*
-              (make-debug-name-marker)))
-        ((eq marker *debug-name-ellipsis*)
-         `(if (boundp '*debug-name-ellipsis*)
-              *debug-name-ellipsis*
-              (make-debug-name-marker)))
-        (t
-         (warn "Dumping unknown debug-name marker.")
-         '(make-debug-name-marker))))
+(define-load-time-global *debug-name-sharp* (make-debug-name-marker))
+(define-load-time-global *debug-name-ellipsis* (make-debug-name-marker))
 
 (defun print-debug-name-marker (marker stream level)
   (declare (ignore level))
@@ -263,9 +260,6 @@ the stack without triggering overflow protection.")
         (t
          (write-string "???" stream))))
 
-(setf *debug-name-sharp* (make-debug-name-marker)
-      *debug-name-ellipsis* (make-debug-name-marker))
-
 (declaim (ftype (sfunction () list) name-context))
 (defun debug-name (type thing &optional context)
   (let ((*debug-name-punt* nil))
@@ -304,7 +298,7 @@ the stack without triggering overflow protection.")
 (defvar *compile-time-eval* nil)
 (declaim (always-bound *compile-time-eval*))
 
-#!-immobile-code (defmacro code-immobile-p (thing) `(progn ,thing nil))
+#-immobile-code (defmacro code-immobile-p (thing) `(progn ,thing nil))
 
 ;;; Various error-code generating helpers
 (defvar *adjustable-vectors*)
@@ -332,7 +326,7 @@ the stack without triggering overflow protection.")
 
 ;;; The allocation quantum for boxed code header words.
 ;;; 2 implies an even length boxed header; 1 implies no restriction.
-(defvar code-boxed-words-align (+ 2 #!+(or x86 x86-64) -1))
+(defconstant code-boxed-words-align (+ 2 #+(or x86 x86-64) -1))
 
 ;;; Used as the CDR of the code coverage instrumentation records
 ;;; (instead of NIL) to ensure that any well-behaving user code will
@@ -355,7 +349,39 @@ the stack without triggering overflow protection.")
     (list (make-hash-table :test 'equal :synchronized t)))
   (declaim (type (cons hash-table) *code-coverage-info*)))
 
-(in-package "SB!ALIEN")
+(defstruct (compilation (:copier nil)
+                        (:predicate nil)
+                        (:conc-name ""))
+  (fun-names-in-this-file)
+  (coverage-metadata nil :type (or (cons hash-table hash-table) null) :read-only t)
+  (msan-unpoison nil :read-only t)
+  (sset-counter 1 :type fixnum)
+  ;; Bidrectional map between IR1/IR2/assembler abstractions
+  ;; and a corresponding small integer identifier. One direction could be done
+  ;; by adding the integer ID as an object slot, but we want both directions.
+  ;; These could just as well be scoped by WITH-IR1-NAMESPACE, but
+  ;; since it's primarily a debugging tool, it's nicer to have
+  ;; a wider unique scope by ID.
+  (objmap-obj-to-id   (make-hash-table :test 'eq) :read-only t)
+  (objmap-id-to-cont  (make-array 10) :type simple-vector) ; number -> CTRAN or LVAR
+  (objmap-id-to-tn    (make-array 10) :type simple-vector) ; number -> TN
+  (objmap-id-to-label (make-array 10) :type simple-vector) ; number -> LABEL
+  (objmap-cont-num    0 :type fixnum)
+  (objmap-tn-id       0 :type fixnum)
+  (objmap-label-id    0 :type fixnum)
+  ;; if emitting a cfasl, the fasl stream to that
+  (compile-toplevel-object nil :read-only t)
+  ;; these are all historical baggage from here down,
+  ;; unused unless we ever decide to fix block compilation
+  (block-compile nil :type (member nil t :specified))
+  ;; When block compiling, used by PROCESS-FORM to accumulate top level
+  ;; lambdas resulting from compiling subforms. (In reverse order.)
+  (toplevel-lambdas nil :type list))
+
+(defvar *compilation*)
+(declaim (type compilation *compilation*))
+
+(in-package "SB-ALIEN")
 
 ;;; Information describing a heap-allocated alien.
 (def!struct (heap-alien-info (:copier nil))
@@ -369,5 +395,5 @@ the stack without triggering overflow protection.")
 
 ;; from 'llvm/projects/compiler-rt/lib/msan/msan.h':
 ;;  "#define MEM_TO_SHADOW(mem) (((uptr)(mem)) ^ 0x500000000000ULL)"
-#!+linux ; shadow space differs by OS
-(defconstant sb!vm::msan-mem-to-shadow-xor-const #x500000000000)
+#+linux ; shadow space differs by OS
+(defconstant sb-vm::msan-mem-to-shadow-xor-const #x500000000000)
