@@ -466,15 +466,15 @@
 (defmethod print-object ((obj interpreted-function) stream)
   ;; Do not try to directly print 'NAME-FOR-FUN', which returns OBJ
   ;; itself if it has no proper name.
-  (let ((name (fun-name obj)))
-    (if (eql name 0)
-        ;; To avoid an extra space between type and identity, the body must
-        ;; be empty, so we need two cases, because emptiness is compile-time
-        ;; determined, not based on whether the body actually printed anything.
-        (print-unreadable-object (obj stream :type t :identity t))
-        ;; show name whenever NAME it is not 0, even if not OBJ's proper name.
-        (print-unreadable-object (obj stream :type t)
-          (prin1 name stream)))))
+  (if (or (eql (interpreted-function-%proto-fn obj) 0)
+          (eql (fun-name obj) 0))
+      ;; To avoid an extra space between type and identity, the body must
+      ;; be empty, so we need two cases, because emptiness is compile-time
+      ;; determined, not based on whether the body actually printed anything.
+      (print-unreadable-object (obj stream :type t :identity t))
+      ;; show name whenever NAME it is not 0, even if not OBJ's proper name.
+      (print-unreadable-object (obj stream :type t)
+        (prin1 (fun-name obj) stream))))
 
 ;;; Return approximately a type specifier for LAMBDA-LIST.
 ;;; e.g. after doing (DEFUN FOO (A B) ...), you want (FUNCTION (T T) *)
@@ -517,7 +517,7 @@
     ;; Hooking all functions, makes them somewhat slower,
     ;; but allows for really nifty introspection,
     ;; such as discovering what calls are made by read-time evals.
-    (setf (funcallable-instance-fun function)
+    (setf (%funcallable-instance-fun function)
           (if *hook-all-functions-p*
               (lambda (&rest args)
                 (apply #'interpreter-hooked-trampoline function args))
@@ -702,7 +702,7 @@
        (multiple-value-bind (definition index)
            (%find-position fname (the simple-vector (env-payload env))
                            nil 0 nil #'fname test)
-         (when definition
+         (when index
            (return (values (if (macro-env-p env) :macro :function)
                            definition (make-frame-ptr index level)))))))))
 
@@ -979,7 +979,7 @@
                (map 'vector
                     (lambda (cell)
                       (let ((expander (cddr cell)))
-                        (if (interpreted-function-p expander)
+                        (if (typep expander 'interpreted-function)
                             expander
                             (make-function
                              (%make-proto-fn `(macrolet ,(car cell)) '(form env)
@@ -1044,7 +1044,7 @@
 ;;; In practice it's likely that an interpreted closure would be "too complex"
 ;;; for other reasons, usually due to surrounding BLOCK. It would be somewhat
 ;;; nifty to walk the code and find that the block is never used.
-(defun prepare-for-compile (function &aux nullify-lexenv)
+(defun sb-c::prepare-for-compile (function &aux nullify-lexenv)
   (if (named-let too-complex-p ((env (interpreted-function-env function)))
         (when (null env)
           (return-from too-complex-p nil))
@@ -1202,7 +1202,7 @@
                       (let ((defined-fun
                              (sb-c::make-defined-fun
                               :%source-name fname
-                              :type (sb-int:proclaimed-ftype fname))))
+                              :type (sb-int:global-ftype fname))))
                         (setf (sb-c::defined-fun-inlinep defined-fun) inlinep)
                         (push (cons fname defined-fun) funs))))))))
             (ftype
