@@ -275,10 +275,10 @@ allocation_trap_p(os_context_t * context)
     unsigned __attribute__((unused)) dst;
 
     /*
-     * First, the instruction has to be a TWLGE temp, NL3, which has the
+     * First, the instruction has to be a TWLGT temp, NL3, which has the
      * format.
-     * | 6| 5| 5 | 5 | 10|1|  width
-     * |31|5 |dst|src|  4|0|  field
+     * | 6| 5| 5 | 5 | 10|1|  field width
+     * |31| 1|dst|src|  4|0|  value
      */
     pc = (unsigned int *) (*os_context_pc_addr(context));
     inst = *pc;
@@ -290,7 +290,7 @@ allocation_trap_p(os_context_t * context)
     opcode = inst >> 26;
     src = (inst >> 11) & 0x1f;
     dst = (inst >> 16) & 0x1f;
-    if ((opcode == 31) && (src == reg_NL3) && (5 == ((inst >> 21) & 0x1f))
+    if ((opcode == 31) && (src == reg_NL3) && (1 == ((inst >> 21) & 0x1f))
         && (4 == ((inst >> 1) & 0x3ff))) {
         /*
          * We got the instruction.  Now, look back to make sure it was
@@ -318,7 +318,9 @@ allocation_trap_p(os_context_t * context)
     return 0;
 }
 
+#ifndef boxed_region
 #define boxed_region gc_alloc_region[0]
+#endif
 
 void
 handle_allocation_trap(os_context_t * context)
