@@ -630,7 +630,7 @@ static boolean instrumentp(uword_t* sp, uword_t** pc, uword_t* old_word)
 
 // logical index 'index' in the metadata array stores the code component
 // and pc-offset relative to the component base address
-static void record_pc(uword_t* pc, unsigned int index, boolean sizedp)
+static void record_pc(char* pc, unsigned int index, boolean sizedp)
 {
     lispobj *code = component_ptr_from_pc(pc);
     if (!code) {
@@ -649,7 +649,7 @@ static void record_pc(uword_t* pc, unsigned int index, boolean sizedp)
         ensure_region_closed(code_region);
         int ret = thread_mutex_unlock(&free_pages_lock);
         gc_assert(ret == 0);
-        code = component_ptr_from_pc((lispobj*)pc);
+        code = component_ptr_from_pc(pc);
     }
 #endif
     struct vector* v = VECTOR(alloc_profile_data);
@@ -685,7 +685,7 @@ allocation_tracker_counted(uword_t* sp)
         if (!__sync_bool_compare_and_swap(pc, word_at_pc, new_inst))
             lose("alloc profiler failed to rewrite instruction @ %p", pc);
         if (index != 2)
-            record_pc(pc, index, 0);
+            record_pc((char*)pc, index, 0);
     }
     int __attribute__((unused)) ret = thread_mutex_unlock(&alloc_profiler_lock);
     gc_assert(ret == 0);
@@ -724,7 +724,7 @@ allocation_tracker_sized(uword_t* sp)
             !__sync_bool_compare_and_swap(pc,   word_at_pc,    new_inst1))
             lose("alloc profiler failed to rewrite instructions @ %p", pc);
         if (index != 0) // can't record a PC for the overflow counts
-            record_pc(pc, index, 1);
+            record_pc((char*)pc, index, 1);
     }
     int __attribute__((unused)) ret = thread_mutex_unlock(&alloc_profiler_lock);
     gc_assert(ret == 0);
